var sys = require("sys"),
    fs = require("fs"),
    path = require("path"),
    url = require("url"),
    cradle = require("cradle"),
    nomnom = require("nomnom"),
    _ = require("underscore")._,
    brain = require("../../lib/brain");

function getDb(couchUrl) {
  var parts = url.parse(couchUrl);
  var client = new cradle.Connection(parts.hostname, parts.port || 80);
  return client.database(parts.pathname);  
}

function getDocs(couchUrl, callback) {
  var db = getDb(couchUrl);
  db.all({include_docs: true}, function(err, res) {
    if(err)
      sys.puts("error retreiving data from " + couchUrl + ": '" + err + "'");
    else {
      var data = _(res.rows).pluck("doc")
      callback(data);
    }
  });
}

function crossValidate(type, options, data, slices) {
  var constructor;
  if(type == "neuralnetwork")
    constructor = brain.NeuralNetwork;
  else if(type == "bayesian")
    constructor = brain.BayesianClassifier;
  
  return brain.crossValidate(constructor, options, data, slices);
}

function runTest(config) {
  var type = config.type || "neuralnetwork";
  var opts = config.options || {};
  var slices = config.slices || 3;
  
  getDocs(config.db, function(data) {
    sys.puts("\nrunning " + type + " test on data size: " + data.length)
    var stats = crossValidate(type, opts, data, slices);
    if(options.verbose)
      sys.inspect(stats);

    var err = 0, train = 0, test = 0;
    var sums = stats.forEach(function(stat) {
      err += stat.error;
      train += stat.trainTime;
      test += stat.testTime;
    });
    sys.puts("\naverage error: " + (err / stats.length));
    sys.puts("average train time: " + (train / stats.length) / 1000 + " s");
    sys.puts("average test time: " + (test / stats.length) + " ms");
    
    if(options.report) {
      var db = getDb(options.report);
      var report = {
        stats: stats,
        name: options.reportName,
        timestamp: new Date(),
        config: config
      }
      db.insert(report, function(err, res) {
        if(err)
          sys.puts("error sending report to " + option.report);
        else
          sys.puts("saved report " + options.report + "/" + res.id);
      }); 
    }
  });
}

var opts = [
  { name: 'target',
    position: 0,
    help: "[neuralnetwork|bayesian]"
  },
  
  { name: 'config',
    string: '-c FILE, --config=FILE',
    default: path.join(__dirname, "cvtests.json"),
    help: 'JSON manifest of cross-validation tests to run'
  },
  
  { string: '-d URL, --db=URL',
    help: 'url to CouchDB database of training data'
  },
  
  { string: '-o JSON, --options=JSON',
    help: 'options to pass to classifier'
  },

  { string: '-t [neuralnetwork|bayesian], --type=TYPE',
    help: 'type of classifier/network to test'
  },

  { name: 'verbose',
    string: '-v, --verbose',
    help: 'print more messages'
  },
  
  { name: 'report',
    string: '-r COUCHDB, --report=COUCHDB',
    help: 'couch db to post results to'
  },
  
  { name: 'reportName',
    string: '-n NAME, --report-name=NAME',
    help: 'name of results report'
  },
];

var options = nomnom.parseArgs(opts, {script: 'node cvtests.js'});

var tests;
if(options.db) {
  tests = [options];
}
else {
  var config = JSON.parse(fs.readFileSync(options.config, "utf-8"));
  if(options.target)
    tests = _(config[options.target]).map(function(test) {
      test.type = options.target;
      return test;
    });
  else
    tests = _(config).reduce(function(allTests, tests, type) {
      tests = tests.map(function(test) {
        test.type = type;
        return test;
      });
      return allTests.concat(tests);
    }, []);
}

_(tests).map(function(testConfig) {
  runTest(testConfig);
});